@@ -1,6 +1,6 @@
 package Myriad::Subscription::Implementation::Redis;
 
-our $VERSION = '0.002'; # VERSION
+our $VERSION = '0.003'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
 use Myriad::Class extends => qw(IO::Async::Notifier);
@@ -44,40 +44,46 @@ async method create_from_source (%args) {
 
     my $stream = "service.subscriptions.$service/$args{channel}";
 
-    $src->unblocked->then(sub {
+    $src->unblocked->then($self->$curry::weak(async method {
         # The streams will be checked later by "check_for_overflow" to avoid unblocking the source by mistake
         # we will make "check_for_overflow" aware about this stream after the service has started
-        push @emitters, {stream => $stream, source => $src, max_len => $args{max_len} // MAX_ALLOWED_STREAM_LENGTH};
-        return $src->map(sub {
+        push @emitters, {
+            stream => $stream,
+            source => $src,
+            max_len => $args{max_len} // MAX_ALLOWED_STREAM_LENGTH
+        };
+        await $src->map($self->$curry::weak(method {
             $log->tracef('sub has an event! %s', $_);
             return $redis->xadd(
                 encode_utf8($stream) => '*',
                 data => encode_json_utf8($_),
             );
-        })->ordered_futures(
+        }))->ordered_futures(
             low => 100,
             high => 5000,
         )->completed
-         ->on_fail(sub {
+         ->on_fail($self->$curry::weak(method {
             $log->warnf("Redis XADD command failed for stream %s", $stream);
             $should_shutdown->fail(
                 "Failed to publish subscription data for $stream - " . shift
             ) unless $should_shutdown->is_ready;
-        })
-    })->retain;
+        }));
+        return;
+    }))->retain;
     return;
 }
 
 async method create_from_sink (%args) {
-    my $sink = delete $args{sink} or die 'need a sink';
+    my $sink = delete $args{sink}
+        or die 'need a sink';
     my $remote_service = $args{from} || $args{service};
     my $stream = "service.subscriptions.$remote_service/$args{channel}";
     $log->tracef('created sub thing from sink');
     push @receivers, {
-        key => $stream,
+        key    => $stream,
         client => $args{client},
-        sink => $sink,
-        group => 0,
+        sink   => $sink,
+        group  => 0,
     };
 }
 
@@ -92,7 +98,8 @@ async method start {
 }
 
 async method stop {
-    $should_shutdown->done() unless $should_shutdown->is_ready;
+    $should_shutdown->done unless $should_shutdown->is_ready;
+    return;
 }
 
 
@@ -101,6 +108,7 @@ async method create_group($receiver) {
         await $redis->create_group($receiver->{key}, $uuid);
         $receiver->{group} = 1;
     }
+    return;
 }
 
 async method receive_items {
@@ -128,20 +136,30 @@ async method receive_items {
             await $self->create_group($item);
 
             my @events = await $redis->read_from_stream(
-                    stream => $stream,
-                    group => $uuid,
-                    client => $client
+                stream => $stream,
+                group => $uuid,
+                client => $client
             );
 
             for my $event (@events) {
                 try {
                     my $event_data = $event->{data}->[1];
-                    $sink->source->emit({data => decode_json_utf8($event_data)});
+                    $sink->source->emit({
+                        data => decode_json_utf8($event_data)
+                    });
                 } catch($e) {
-                    $log->tracef("An error happned while decoding event data for stream %s message: %s , error: %s",
-                    $stream, $event->{data}, $e);
+                    $log->tracef(
+                        "An error happened while decoding event data for stream %s message: %s, error: %s",
+                        $stream,
+                        $event->{data},
+                        $e
+                    );
                 }
-                await $redis->ack($stream, $client, $event->{id});
+                await $redis->ack(
+                    $stream,
+                    $client,
+                    $event->{id}
+                );
             }
         } else {
             $log->tracef('No receivers, waiting for a few seconds');
@@ -150,7 +168,6 @@ async method receive_items {
     }
 }
 
-
 async method check_for_overflow () {
     while (1) {
         if(@emitters) {
@@ -163,7 +180,10 @@ async method check_for_overflow () {
                         $emitter->{source}->pause;
                         $log->tracef("Paused emitter on %s, length is %s, max allowed %s", $emitter->{stream}, $len, $emitter->{max_len});
                     }
-                    await $redis->cleanup(stream => $emitter->{stream}, limit => $emitter->{max_len});
+                    await $redis->cleanup(
+                        stream => $emitter->{stream},
+                        limit => $emitter->{max_len}
+                    );
                 } else {
                     if($emitter->{source}->is_paused) {
                         $emitter->{source}->resume;
